import {
  mapURL,
  EVDataURL,
  width,
  mapWidth,
  plotWidth,
  height,
  margin_top,
  margin_else,
} from './globals.js';

// map
export const svgMap = d3
  .select('#nshields_dataviz')
  .append('svg')
  .attr('width', mapWidth)
  .attr('height', height);

// plot
export const svgPlot = d3
  .select('#nshields_dataviz')
  .append('svg')
  .attr('width', plotWidth)
  .attr('height', height)
  .attr('id', 'svgPlot');

// tooltip
export var toolTip = d3
  .select('#nshields_dataviz')
  .append('div')
  .attr('id', 'ToolTip');

// map legend gradient
export var gradientColors = [
  'rgb(30, 105, 5)',
  'rgb(255, 255, 255)',
];
export var svgGradient = d3
  .select('body')
  .append('svg')
  .attr('width', 100)
  .attr('height', 200);

export var gradient = svgGradient
  .append('defs')
  .append('linearGradient')
  .attr('id', 'gradient')
  .attr('x1', '0%')
  .attr('x2', '0%')
  .attr('y1', '0%')
  .attr('y2', '100%');
//
//
//
// clip
export var clip = svgPlot
  .append('defs')
  .append('svg:clipPath')
  .attr('id', 'clip')
  .append('svg:rect')
  .attr('width', width)
  .attr('height', height)
  .attr('x', 0)
  .attr('y', 0);

// scatter var
export var scatter = svgPlot
  .append('g')
  .attr('clip-path', '#url(#clip)');

// set projection
export function projectionSetUp(mapData) {
  var projection = d3
    .geoMercator()
    .fitSize([width / 2.2, height], mapData);
  return projection;
}
//
//
//

// x values
export function xValues(countyData, xAttr) {
  var x = d3
    .scaleLinear()
    .domain(
      d3.extent(
        d3.map(
          countyData.values().filter((d) => d.County),
          (d) => +d[xAttr],
        ),
      ),
    )
    .range([margin_else, plotWidth - margin_else]);
  return x;
}

//y values
export function yValues(countyData, yAttr) {
  var y = d3
    .scaleLinear()
    .domain(
      d3.extent(
        d3.map(
          countyData.values().filter((d) => d.County),
          (d) => +d[yAttr],
        ),
      ),
    )
    .range([height - margin_else, margin_top]);
  return y;
}

export function rScaleSetUp(countyData, rAttr) {
  const rScale = d3
    .scaleLinear()
    .domain(
      d3.extent(
        d3.map(countyData.values(), (d) => d[rAttr]),
      ),
    )
    .range([4, 10]);
  return rScale;
}

export function xAxisSetUp(x) {
  var xAxis = svgPlot
    .append('g')
    .attr(
      'transform',
      `translate(0,${height - margin_else})`,
    )
    .call(d3.axisBottom(x));
  return xAxis;
}

export function yAxisSetUp(y) {
  var yAxis = svgPlot
    .append('g')
    .attr('transform', `translate(${margin_else},0)`)
    .call(d3.axisLeft(y));
  return yAxis;
}

// x axis label
svgPlot
  .append('text')
  .attr('text-anchor', 'end')
  .attr('x', plotWidth - 40)
  .attr('y', height - 15)
  .text('EV Count')
  .style('font-size', '16px')
  .style('font-family', 'Helvetica');

// y axis label
svgPlot
  .append('text')
  .attr('text-anchor', 'end')
  .attr('transform', 'rotate(-90)')
  .attr('x', -10)
  .attr('y', 16)
  .text('Real GDP $ Per Capita')
  .style('font-size', '16px')
  .style('font-family', 'Helvetica');

// brush popup
svgPlot
  .append('text')
  .attr('text-anchor', 'end')
  .attr('x', 140)
  .attr('y', height - 16)
  .text('*Click + drag to zoom')
  .style('font-size', '10px')
  .style('font-family', 'Helvetica')
  .style('font-style', 'italic');
svgPlot
  .append('text')
  .attr('text-anchor', 'end')
  .attr('x', 240)
  .attr('y', height - 16)
  .text('*Double click to reset')
  .style('font-size', '10px')
  .style('font-family', 'Helvetica')
  .style('font-style', 'italic');

// legend for plot
var legendposition_pointx = 500;
var legendposition_pointy = 420;
var legendposition_text = legendposition_pointx - 45;

svgPlot
  .append('circle')
  .attr('cx', legendposition_pointx)
  .attr('cy', legendposition_pointy)
  .attr('r', 4)
  .style('fill', 'tan')
  .attr('stroke', 'black')
  .attr('stroke-width', 0.5);
svgPlot
  .append('text')
  .attr('x', legendposition_text)
  .attr('y', legendposition_pointy + 2)
  .text('Rural')
  .style('font-size', '14px')
  .attr('alignment-baseline', 'middle')
  .style('font-family', 'Helvetica');
svgPlot
  .append('circle')
  .attr('cx', legendposition_pointx)
  .attr('cy', legendposition_pointy + 16)
  .attr('r', 4)
  .style('fill', 'steelblue')
  .attr('stroke', 'black')
  .attr('stroke-width', 0.5);
svgPlot
  .append('text')
  .attr('x', legendposition_text)
  .attr('y', legendposition_pointy + 18)
  .text('Urban')
  .style('font-size', '14px')
  .attr('alignment-baseline', 'middle')
  .style('font-family', 'Helvetica');

export function mapColorScaleSetUp(
  countyData,
  mapColorAttr,
) {
  var mapColorScale = d3
    .scaleLinear()
    .domain(
      d3.extent(
        d3.map(
          countyData.values(),
          (r) => +r[mapColorAttr],
        ),
      ),
    )
    .range(['white', 'green']);

  return mapColorScale;
}

// map legend with gradient
gradient
  .selectAll('stop')
  .data(gradientColors)
  .enter()
  .append('stop')
  .style('stop-color', function (d) {
    return d;
  })
  .attr('offset', function (d, i) {
    return 100 * (i / (gradientColors.length - 1)) + '%';
  });
var yposition = 400;
svgMap
  .append('rect')
  .attr('x', 360)
  .attr('y', yposition)
  .attr('width', 15)
  .attr('height', 15)
  .style('fill', 'url(#gradient)')
  .attr('stroke', 'black')
  .attr('stroke-width', 0.5);
svgMap
  .append('text')
  .attr('x', 380)
  .attr('y', yposition + 10)
  .text('EV Count Per Capita')
  .style('font-size', '14px')
  .attr('alignment-baseline', 'middle')
  .style('font-family', 'Helvetica');
